const Hp = require('hemera-plugin');
import { NatsPubSub } from 'graphql-nats-subscriptions';
import { Container } from 'inversify';
import * as ILogger from 'bunyan';
import { OutputCommands, TYPES, NATS_HEMERA_KUBE_OUTPUT} from '../constants';
import { outputModule } from '../container';
import {  IOutputSettings, IOutputService } from '../interfaces';

import {setupProjects} from '../parser';

function hemeraPreferences(hemera, options: { settings: IOutputSettings, client: any }, done) {
    const { settings, client } = options;

    const topic = `${NATS_HEMERA_KUBE_OUTPUT}/${settings.subTopic}`;

    const pubsub = new NatsPubSub({ client, logger: hemera.log });

    let container = new Container();
    container.bind<ILogger>('Logger').toConstantValue(hemera.log);
    container.bind('Settings').toConstantValue({...settings}).whenTargetTagged('microservices', true);
    container.bind('Settings').toConstantValue({...settings}).whenTargetTagged('default', true);
    container.bind('PubSub').toConstantValue(pubsub);
    container.load(outputModule(settings));

    const service = container.get<IOutputService>(TYPES.IOutputService);

    setupProjects(service, hemera.log, settings.podConfig);
    hemera.add({
        topic,
        cmd: OutputCommands.ADD_OUTPUT_POD,
    }, async (args) => {
        return service.addNewType(args.podName, args.newItem);
    });

    hemera.add({
        topic,
        cmd: OutputCommands.GET_OUTPUT_PODS,
    }, async (args) => {
        return service.getPods();
    });

    hemera.add({
        topic,
        cmd: OutputCommands.GET_OUTPUT_DATA,
    }, async (args) => {
        return service.getOutput(args.request);
    });

    hemera.add({
        topic,
        cmd: OutputCommands.UNWATCH_OUTPUT,
    }, async () => {
        hemera.log.debug('GET_WORKSPACE_PREF_VALUE: (%j)');
        return service.unWatchLogs();
    });

    hemera.add({
        topic,
        cmd: OutputCommands.DISPOSE_OUTPUT,
    }, async (args) => {
        hemera.log.debug('dispose output: (%j)', args);
        return service.dispose();
    });
}

module.exports = Hp(hemeraPreferences, {
    hemera: '>=2.0.0-0',
    name: require('../../package.json').name,
});
