import { injectable, inject, named, tagged } from 'inversify';
import * as Logger from 'bunyan';
import * as Hemera from 'nats-hemera';
import { TaggedType } from '@common-stack/core';
import { IOutputService, IPodsRes, IOutputSettings, INewOutputPod } from '../interfaces';
import { OutputCommands, TYPES, NATS_HEMERA_KUBE_OUTPUT } from '../constants';

@injectable()
export class OutputMicroservice implements IOutputService {
    constructor(
        @inject('Hemera') private hemera: Hemera<Request, Response>,
        @inject(TYPES.IOutputSettings)
        @named(TaggedType.MICROSERVICE) private settings: IOutputSettings,
    ) {}
    private topic = `${NATS_HEMERA_KUBE_OUTPUT}/${this.settings.apiSubTopic}`;

    public addNewType(podName: string, newItem: INewOutputPod): void {
        return this.hemera.act({
            topic: this.topic,
            cmd: OutputCommands.ADD_OUTPUT_POD,
            podName,
            newItem,
        }) as any;
    }

    public getPods(): IPodsRes[] {
        return this.hemera.act({
            topic: this.topic,
            cmd: OutputCommands.GET_OUTPUT_PODS,
        }) as any;
    }

    public getOutput(request: string) {
        return this.hemera.act({
            topic: this.topic,
            cmd: OutputCommands.GET_OUTPUT_DATA,
            request,
        }) as any;
    }

    public unWatchLogs(): Boolean {
        return this.hemera.act({
            topic: this.topic,
            cmd: OutputCommands.UNWATCH_OUTPUT,
        }) as any;
    }

    public dispose() {
        return this.hemera.act({
            topic: this.topic,
            cmd: OutputCommands.DISPOSE_OUTPUT,
        }) as any;
    }

}
